import com.sun.java.swing.*;

import footfall.displays.*;
import footfall.streams.*;
import footfall.synths.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.util.Vector;

public class FootFallDriver extends JApplet
{
  FootFallStream footfallstream;
  FootFallStreamFile footfallstreamfile = new FootFallStreamFile("2004-03-19.bar");
  FootFallStreamGen footfallstreamgen = new FootFallStreamGen();

  FootFallSynth footfallsynth;
  FootFallSynthByFrame footfallsynthbyframe = new FootFallSynthByFrame();
  FootFallSynthByCell footfallsynthbycell = new FootFallSynthByCell();

  DisplayFootFall displayfootfall = new DisplayFootFall();
  DisplaySound displaysound = new DisplaySound();

  public FootFallDriver()
  {
    footfallstream = footfallstreamfile;
    footfallstream.setDisplay(displayfootfall);

    footfallsynth = footfallsynthbyframe;
    footfallsynth.setDisplay(displaysound);
    footfallsynth.setFootFallStream(footfallstream);

    JInternalFrame right = new JInternalFrame();
      right.getContentPane().add(new DisplayFootFallControls(this), "Center");
      right.getContentPane().add(new FootFallSynthControls(this), "South");

    getContentPane().add(displayfootfall, "Center");
    getContentPane().add(right.getContentPane(), "East");
    getContentPane().add(displaysound, "South");
  }

  public void setFootFallStream(FootFallStream footfallstream)
  {
    stop();

    this.footfallstream = footfallstream;
    footfallsynth.setFootFallStream(this.footfallstream);

    this.footfallstream.setDisplay(this.displayfootfall);
    footfallsynth.setDisplay(this.displaysound);

    start();
  }

  public void setFootFallSynth(FootFallSynth footfallsynth)
  {
    this.footfallsynth.stop();

    this.footfallsynth = footfallsynth;
    footfallsynth.setDisplay(this.displaysound);
    footfallsynth.setFootFallStream(this.footfallstream);

    this.footfallsynth.start();
  }

  public void start()
  {
    this.footfallstream.start();
    this.footfallsynth.start();
  }

  public void stop()
  {
    this.footfallstream.stop();
    this.footfallsynth.stop();
  }

  static class DisplayFootFallControls extends JPanel implements ActionListener, AdjustmentListener
  {
    FootFallDriver footfalldriver;
    FootFallStream footfallstream;
    JScrollBar fpsScroll, weightScroll;
    Font font = new Font("sansserif", Font.PLAIN, 11);

    public DisplayFootFallControls(FootFallDriver footfalldriver)
    {
      this.footfalldriver = footfalldriver;
      this.footfallstream = footfalldriver.footfallstream;
      setLayout(new BorderLayout());
      JToolBar bar = new JToolBar(JToolBar.VERTICAL);
      bar.setFloatable(false);

      fpsScroll = new JScrollBar(SwingConstants.HORIZONTAL, footfallstream.getFramesPerSecond(), 0, footfallstream.getFramesPerSecondMin(), footfallstream.getFramesPerSecondMax());
      fpsScroll.addAdjustmentListener(this);
      TitledBorder tb = new TitledBorder(new EtchedBorder());
      tb.setTitleFont(font);
      tb.setTitle(String.valueOf(footfallstream.getFramesPerSecond()) + " fps : Running Speed (Turtle = 25 - Hare = 75)");
      fpsScroll.setBorder(tb);
      fpsScroll.setPreferredSize(new Dimension(360,44));
      bar.add(fpsScroll);

      weightScroll = new JScrollBar(SwingConstants.HORIZONTAL, 1, 0, 1, 50);
      weightScroll.addAdjustmentListener(this);
      tb = new TitledBorder(new EtchedBorder());
      tb.setTitleFont(font);
      tb.setTitle("10 lbs Body Weight (Baby = 10 - Sumo Wrestler = 500)");
      weightScroll.setBorder(tb);
      weightScroll.setPreferredSize(new Dimension(360,44));
      bar.add(weightScroll);

      JToolBar btnbar = new JToolBar(JToolBar.HORIZONTAL);
      btnbar.setFloatable(false);

      JButton b = new JButton("Recorded Data");
      b.setAlignmentX(0.5f);
      b.setFont(font);
      b.setToolTipText("File: 2004-03-19.bar");
      b.addActionListener(this);
      btnbar.add(b);

      b = new JButton("Lazy Walk");
      b.setAlignmentX(0.5f);
      b.setFont(font);
      b.setToolTipText("");
      b.addActionListener(this);
      btnbar.add(b);

      b = new JButton("Jog");
      b.setAlignmentX(0.5f);
      b.setFont(font);
      b.setToolTipText("");
      b.addActionListener(this);
      btnbar.add(b);

      b = new JButton("Run");
      b.setAlignmentX(0.5f);
      b.setFont(font);
      b.setToolTipText("");
      b.addActionListener(this);
      btnbar.add(b);

      b = new JButton("Light Walk");
      b.setAlignmentX(0.5f);
      b.setFont(font);
      b.setToolTipText("");
      b.addActionListener(this);
      btnbar.add(b);

      b = new JButton("Heavy Walk");
      b.setAlignmentX(0.5f);
      b.setFont(font);
      b.setToolTipText("");
      b.addActionListener(this);
      btnbar.add(b);

      tb = new TitledBorder(new EtchedBorder());
      tb.setTitleFont(font);
      tb.setTitle("Kinematic Presets");
      btnbar.setBorder(tb);

      add(btnbar, "North");
      add(bar, "Center");
    }

    public void actionPerformed(ActionEvent e)
    {
      JButton b = (JButton) e.getSource();

      if(b.getText().equals("Recorded Data"))
      {
        if(!(this.footfallstream instanceof FootFallStreamFile))
        {
          this.footfallstream = this.footfalldriver.footfallstreamfile;
          this.footfalldriver.setFootFallStream(this.footfallstream);
        }
      }
      else
      {
        if(!(this.footfallstream instanceof FootFallStreamGen))
        {
          this.footfallstream = this.footfalldriver.footfallstreamgen;
          this.footfalldriver.setFootFallStream(this.footfallstream);
        }
      }

      if(b.getText().equals("Recorded Data"))
      {
        fpsScroll.setValue(50);
        weightScroll.setValue(1);
      }
      else if (b.getText().equals("Lazy Walk"))
      {
        fpsScroll.setValue(30);
        weightScroll.setValue(20);
      }
      else if(b.getText().equals("Jog"))
      {
        fpsScroll.setValue(60);
        weightScroll.setValue(25);
      }
      else if(b.getText().equals("Run"))
      {
        fpsScroll.setValue(70);
        weightScroll.setValue(30);
      }
      else if(b.getText().equals("Light Walk"))
      {
        fpsScroll.setValue(55);
        weightScroll.setValue(15);
      }
      else if(b.getText().equals("Heavy Walk"))
      {
        fpsScroll.setValue(45);
        weightScroll.setValue(45);
      }
    }

    public void adjustmentValueChanged(AdjustmentEvent e)
    {
      JScrollBar scrollbar = (JScrollBar) e.getSource();
      int value = scrollbar.getValue();
      TitledBorder tb = (TitledBorder) scrollbar.getBorder();
      if (scrollbar == fpsScroll)
      {
        tb.setTitle(String.valueOf(value) + " fps : Running Speed (Turtle = 25 - Hare = 75)");
        footfallstream.setFramesPerSecond(value);
        scrollbar.repaint();
      }
      else if (scrollbar == weightScroll)
      {
        this.footfallstream.setMaxPressure(value);
        tb.setTitle(String.valueOf((int)(value*10)) + " lbs Body Weight (Baby = 10 - Sumo Wrestler = 500)");
        scrollbar.repaint();
      }
    }
  } // End DisplayFootFallControls class

  static class FootFallSynthControls extends JPanel implements ActionListener, AdjustmentListener
  {
    FootFallDriver footfalldriver;
    JScrollBar crunchinessScroll, compressionScroll, grainsizeScroll;
    JScrollBar carpetScroll, woodennessScroll, creakinessScroll;
    JScrollBar volumeScroll;

    Font font = new Font("sansserif", Font.PLAIN, 11);

    public FootFallSynthControls(FootFallDriver footfalldriver)
    {
      this.footfalldriver = footfalldriver;

      TitledBorder tb;
      setLayout(new BorderLayout());

      JToolBar bar = new JToolBar(JToolBar.VERTICAL);
      bar.setFloatable(false);

      crunchinessScroll = new JScrollBar(SwingConstants.HORIZONTAL, (int)(100*this.footfalldriver.footfallsynth.getParam("crunchiness")), 0, 0, 100);
      crunchinessScroll.addAdjustmentListener(this);
      tb = new TitledBorder(new EtchedBorder());
      tb.setTitleFont(font);
      tb.setTitle(String.valueOf((int)(100*this.footfalldriver.footfallsynth.getParam("crunchiness"))) + ": Crunchiness (Sand = 0 - Snow = 100)");
      crunchinessScroll.setBorder(tb);
      crunchinessScroll.setPreferredSize(new Dimension(360,44));
      bar.add(crunchinessScroll);

      compressionScroll = new JScrollBar(SwingConstants.HORIZONTAL, (int)(100*this.footfalldriver.footfallsynth.getParam("compression")), 0, 0, 100);
      compressionScroll.addAdjustmentListener(this);
      tb = new TitledBorder(new EtchedBorder());
      tb.setTitleFont(font);
      tb.setTitle(String.valueOf((int)(100*this.footfalldriver.footfallsynth.getParam("compression"))) + ": Compression (Sand = 0 - Snow = 100)");
      compressionScroll.setBorder(tb);
      compressionScroll.setPreferredSize(new Dimension(360,44));
      bar.add(compressionScroll);

      grainsizeScroll = new JScrollBar(SwingConstants.HORIZONTAL, (int)(100*this.footfalldriver.footfallsynth.getParam("grainsize")), 0, 0, 100);
      grainsizeScroll.addAdjustmentListener(this);
      tb = new TitledBorder(new EtchedBorder());
      tb.setTitleFont(font);
      tb.setTitle(String.valueOf((int)(100*this.footfalldriver.footfallsynth.getParam("grainsize"))) + " Grain Size (Solid = 0 - Fine = 100)");
      grainsizeScroll.setBorder(tb);
      grainsizeScroll.setPreferredSize(new Dimension(360,44));
      bar.add(grainsizeScroll);

      carpetScroll = new JScrollBar(SwingConstants.HORIZONTAL, (int)(100*this.footfalldriver.footfallsynth.getParam("carpet")), 0, 0, 100);
      carpetScroll.addAdjustmentListener(this);
      tb = new TitledBorder(new EtchedBorder());
      tb.setTitleFont(font);
      tb.setTitle(String.valueOf((int)(100*this.footfalldriver.footfallsynth.getParam("carpet"))) + " Carpet (Soft = 0 - Hard = 100)");
      carpetScroll.setBorder(tb);
      carpetScroll.setPreferredSize(new Dimension(360,44));
      bar.add(carpetScroll);

      woodennessScroll = new JScrollBar(SwingConstants.HORIZONTAL, (int)(100*this.footfalldriver.footfallsynth.getParam("woodenness")), 0, 0, 100);
      woodennessScroll.addAdjustmentListener(this);
      tb = new TitledBorder(new EtchedBorder());
      tb.setTitleFont(font);
      tb.setTitle(String.valueOf((int)(100*this.footfalldriver.footfallsynth.getParam("woodenness"))) + ": Woodenness (Soft = 0 - Hard = 100)");
      woodennessScroll.setBorder(tb);
      woodennessScroll.setPreferredSize(new Dimension(360,44));
      bar.add(woodennessScroll);

      creakinessScroll = new JScrollBar(SwingConstants.HORIZONTAL, (int)(100*this.footfalldriver.footfallsynth.getParam("creakiness")), 0, 0, 100);
      creakinessScroll.addAdjustmentListener(this);
      tb = new TitledBorder(new EtchedBorder());
      tb.setTitleFont(font);
      tb.setTitle(String.valueOf((int)(100*this.footfalldriver.footfallsynth.getParam("creakiness"))) + ": Creakiness (Solid = 0 - Old Wood = 100)");
      creakinessScroll.setBorder(tb);
      creakinessScroll.setPreferredSize(new Dimension(360,44));
      bar.add(creakinessScroll);

      JToolBar btnbar = new JToolBar(JToolBar.HORIZONTAL);
      btnbar.setFloatable(false);

      JButton b = new JButton("Silence");
      b.setAlignmentX(0.5f);
      b.setFont(font);
      b.setToolTipText("No sounds");
      b.addActionListener(this);
      btnbar.add(b);

      b = new JButton("Sand");
      b.setAlignmentX(0.5f);
      b.setFont(font);
      b.setToolTipText("Plain sand");
      b.addActionListener(this);
      btnbar.add(b);

      b = new JButton("Soft Snow");
      b.setAlignmentX(0.5f);
      b.setFont(font);
      b.setToolTipText("Soft snow with light crunch");
      b.addActionListener(this);
      btnbar.add(b);

      b = new JButton("Hard Snow");
      b.setAlignmentX(0.5f);
      b.setFont(font);
      b.setToolTipText("Very crunchy snow");
      b.addActionListener(this);
      btnbar.add(b);

      b = new JButton("Boardwalk");
      b.setAlignmentX(0.5f);
      b.setFont(font);
      b.setToolTipText("Walking on wooden boardwalk");
      b.addActionListener(this);
      btnbar.add(b);

      b = new JButton("Old Ship");
      b.setAlignmentX(0.5f);
      b.setFont(font);
      b.setToolTipText("Very creaky with thick knocks");
      b.addActionListener(this);
      btnbar.add(b);

      b = new JButton("Sneak-In");
      b.setAlignmentX(0.5f);
      b.setFont(font);
      b.setToolTipText("Midnight Snack Walk");
      b.addActionListener(this);
      btnbar.add(b);

      tb = new TitledBorder(new EtchedBorder());
      tb.setTitleFont(font);
      tb.setTitle("Audio Presets");
      btnbar.setBorder(tb);

      JToolBar volbar = new JToolBar(JToolBar.HORIZONTAL);
      volbar.setFloatable(false);

      volumeScroll = new JScrollBar(SwingConstants.VERTICAL, this.footfalldriver.footfallsynth.getPlayer().getVolume(), 0, 0, 127);
      volumeScroll.addAdjustmentListener(this);
      tb = new TitledBorder(new EtchedBorder());
      tb.setTitleFont(font);
      tb.setTitle("Vol: " + this.footfalldriver.footfallsynth.getPlayer().getVolumeLabel() + "%");
      volumeScroll.setBorder(tb);
      volumeScroll.setPreferredSize(new Dimension(70,1));
      volbar.add(volumeScroll);

      JInternalFrame down = new JInternalFrame();
        down.getContentPane().add(volbar, "East");
        down.getContentPane().add(bar, "Center");

      add(btnbar, "North");
      add(down.getContentPane(), "Center");
    }

    public void actionPerformed(ActionEvent e)
    {
      JButton b = (JButton) e.getSource();

      if(b.getText().equals("Silence"))
      {
        crunchinessScroll.setValue(0);
        compressionScroll.setValue(0);
        grainsizeScroll.setValue(0);
        carpetScroll.setValue(0);
        woodennessScroll.setValue(0);
        creakinessScroll.setValue(0);
      }
      else if (b.getText().equals("Sand"))
      {
        crunchinessScroll.setValue(0);
        compressionScroll.setValue(0);
        grainsizeScroll.setValue(20);
        carpetScroll.setValue(0);
        woodennessScroll.setValue(0);
        creakinessScroll.setValue(0);
      }
      else if(b.getText().equals("Soft Snow"))
      {
        crunchinessScroll.setValue(30);
        compressionScroll.setValue(10);
        grainsizeScroll.setValue(20);
        carpetScroll.setValue(0);
        woodennessScroll.setValue(0);
        creakinessScroll.setValue(0);
      }
      else if(b.getText().equals("Hard Snow"))
      {
        crunchinessScroll.setValue(50);
        compressionScroll.setValue(70);
        grainsizeScroll.setValue(15);
        carpetScroll.setValue(0);
        woodennessScroll.setValue(0);
        creakinessScroll.setValue(0);
      }
      else if(b.getText().equals("Boardwalk"))
      {
        crunchinessScroll.setValue(0);
        compressionScroll.setValue(0);
        grainsizeScroll.setValue(0);
        carpetScroll.setValue(50);
        woodennessScroll.setValue(80);
        creakinessScroll.setValue(0);
      }
      else if(b.getText().equals("Old Ship"))
      {
        crunchinessScroll.setValue(0);
        compressionScroll.setValue(0);
        grainsizeScroll.setValue(10);
        carpetScroll.setValue(80);
        woodennessScroll.setValue(30);
        creakinessScroll.setValue(60);
      }
      else if(b.getText().equals("Sneak-In"))
      {
        crunchinessScroll.setValue(0);
        compressionScroll.setValue(0);
        grainsizeScroll.setValue(0);
        carpetScroll.setValue(20);
        woodennessScroll.setValue(0);
        creakinessScroll.setValue(20);
      }
    }

    public void adjustmentValueChanged(AdjustmentEvent e)
    {
      JScrollBar scrollbar = (JScrollBar) e.getSource();
      int value = scrollbar.getValue();
      TitledBorder tb = (TitledBorder) scrollbar.getBorder();
      if (scrollbar == crunchinessScroll)
      {
        this.footfalldriver.footfallsynth.setParam("crunchiness", value/100f);
        tb.setTitle(String.valueOf(value) + ": Crunchiness (Sand = 0 - Snow = 100)");
        scrollbar.repaint();
      }
      else if (scrollbar == compressionScroll)
      {
        this.footfalldriver.footfallsynth.setParam("compression", value/100f);
        tb.setTitle(String.valueOf(value) + ": Compression (Sand = 0 - Snow = 100)");
        scrollbar.repaint();
      }
      else if (scrollbar == grainsizeScroll)
      {
        this.footfalldriver.footfallsynth.setParam("grainsize", value/100f);
        tb.setTitle(String.valueOf(value) + ": Grain Size (Solid = 0 - Fine = 100)");
        scrollbar.repaint();
      }
      else if (scrollbar == carpetScroll)
      {
        this.footfalldriver.footfallsynth.setParam("carpet", value/100f);
        tb.setTitle(String.valueOf(value) + ": Carpet (Soft = 0 - Hard = 100)");
        scrollbar.repaint();
      }
      else if (scrollbar == woodennessScroll)
      {
        this.footfalldriver.footfallsynth.setParam("woodenness", value/100f);
        tb.setTitle(String.valueOf(value) + ": Woodenness (Soft = 0 - Hard = 100)");
        scrollbar.repaint();
      }
      else if (scrollbar == creakinessScroll)
      {
        this.footfalldriver.footfallsynth.setParam("creakiness", value/100f);
        tb.setTitle(String.valueOf(value) + ": Creakiness (Solid = 0 - Old Wood = 100)");
        scrollbar.repaint();
      }
      else if (scrollbar == volumeScroll)
      {
        this.footfalldriver.footfallsynth.getPlayer().setVolume(value);
        tb.setTitle("Vol: " + this.footfalldriver.footfallsynth.getPlayer().getVolumeLabel() + "%");
        scrollbar.repaint();
      }
    }

  } // End FootFallSynthControls class

  public static void main(String argv[])
  {
    try { UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel"); }
    catch (Exception e) { }

    final FootFallDriver FootFallDriver = new FootFallDriver();

    JFrame f = new JFrame("FootFall Synthesis");
    f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

    f.addWindowListener(new WindowAdapter()
    {
      public void windowDeiconified(WindowEvent e) { FootFallDriver.start(); }
      public void windowIconified(WindowEvent e) { FootFallDriver.stop(); }
    });

    f.getContentPane().add("Center", FootFallDriver);
    f.pack();
    f.setSize(new Dimension(900,640));
    f.setVisible(true);
    FootFallDriver.start();
 }

}