package footfall.displays;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
  Abstract class for displaying footfalls, oscilloscope etc. Can be enabled/disabled by mouse click
  @author Chirag Mehta (chirag@chir.ag)
*/

public abstract class Display extends JPanel implements MouseListener
{
  // Is the display enabled?
  boolean enabled = true;

  /**
    Listen for clicks to enable/disable the display.
  */
  public Display()
  {
    addMouseListener(this);
  }

  /**
    Enable the display. May be processor intensive.
  */
  public void enable()
  {
    this.enabled = true;
  }

  /**
    Disable the display. Save processor cycles.
  */
  public void disable()
  {
    this.enabled = false;
  }

  public void mouseClicked(MouseEvent e) { }
  public void mouseEntered(MouseEvent e) { }
  public void mouseExited(MouseEvent e) { }
  public void mousePressed(MouseEvent e) { }

  /**
    Toggle the display on every click.
    @param e Ignore the parameter
  */
  public void mouseReleased(MouseEvent e)
  {
    toggle();
  }

  /**
    Enable or disable the display by clicking anywhere on the display panel.
  */
  public void toggle()
  {
    if(this.enabled)
      disable();
    else
      enable();
  }
}