package footfall.displays;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import javax.swing.*;

/**
  Display the sound generated by footfalls like an oscilloscope
  @author Chirag Mehta (chirag@chir.ag)
*/

public class DisplaySound extends Display
{
  Graphics g3 = getGraphics();
  BufferedImage bufferedimage;
  Dimension dimension;

  float[] historyBuf;
  int historyCounter = 0;

  // Colors for green old-style oscilloscope
  Color darkC = new Color(0f, 0.2f, 0f);
  Color midC = new Color(0.4f, 0.7f, 0.4f);
  Color lightC = new Color(0.6f, 0.9f, 0.6f);

  /**
    Display the sound generated by footfalls like an oscilloscope. Turned off be default.
  */
  public DisplaySound()
  {
    super();
    setPreferredSize(new Dimension(1,100));
    setBackground(darkC);
    disable();
  }

  /**
    Disable the display. Save processor cycles.
  */
  public void disable()
  {
    this.enabled = false;

    Graphics g = getGraphics();
    if(g != null)
    {
      g.setColor(darkC);
      g.fillRect(0, 0, getSize().width, getSize().height);
      repaint();
    }
  }

  /**
    If enabled, draw the oscilloscope using double-buffering for speed.
    @param x Amplitude of the sound byte
  */
  public void updateDisplay(float[] x)
  {
    if(!this.enabled) return;
    Graphics g = getGraphics();

    if(g != null)
    {
      Graphics2D g2 = (Graphics2D)g;
      int intv = 32;

      if(dimension == null || !getSize().equals(dimension))
      {
        dimension = getSize();
        bufferedimage = (BufferedImage)createImage(dimension.width, dimension.height);

        g3 = bufferedimage.createGraphics();
        g3.setColor(darkC);
        g3.fillRect(0, 0, dimension.width, dimension.height);

        historyBuf = new float[dimension.width];
        historyCounter = 0;
      }

      for(int i = 0; i < x.length; i+=8)
        historyBuf[historyCounter++ % historyBuf.length] = x[i];

      historyCounter = historyCounter % historyBuf.length;

      int y = (int)(dimension.height/2);

      g3.setColor(darkC);
      g3.fillRect(0, 0, getSize().width, getSize().height);
      g3.setColor(midC);

      for(int i = 0; i < historyBuf.length; i++)
        g3.drawLine(i, y, i, (int)(y - historyBuf[(i + historyCounter) % historyBuf.length]*y));

      g3.setColor(lightC);
      g3.drawLine(0, y, dimension.width-1, y);

      // Draws the buffered image to the screen.
      g2.drawImage(bufferedimage, 0, 0, this);

    }
  }
}