package footfall.streams;

import footfall.displays.*;
import footfall.generators.*;

/**
  Generate stream of footfall based on FootModel, speed, and weight.
  @author Chirag Mehta (chirag@chir.ag)
*/

public class FootFallStreamGen extends FootFallStream
{
  // Foot Model
  FootModel leftfoot = new FootModel(0);
  FootModel rightfoot = new FootModel(-getFrameHeight()*3);

  // FootFall data and parameters
  private float[][] footData;

  // FootFall animation frame number
  private int currentFrame = 0;

  /**
    Foot fall stream auto-generator
  */
  public FootFallStreamGen()
  {
    footData = new float[getFrameHeight()][getFrameWidth() * 2];
    setMaxPressure(50);
  }

  /**
    Returns the current frame if it has changed since last call. If frame hasn't changed, returns null.
  */
  public float[][] getCurrentFrame()
  {
    return footData;
  }

  /**
    Returns incremental frame number
  */
  public int getCurrentFrameNum()
  {
    return this.currentFrame;
  }

  /**
    Label for current frame number
  */
  public String getFrameName()
  {
    return "Auto-generated Frame #" + String.valueOf(this.currentFrame);
  }

  /**
    Set the maximum pressure (0-50) for this stream. Weight in lbs is taken as roughly 10x the maximum pressure, hence 10lbs - 500lbs.
  */
  public void setMaxPressure(float fMax)
  {
    this.maxPressure = fMax;
  }

  /**
    Generate the next frame based on footprints, speed, and weight (maximum pressure).
  */
  public void update()
  {
    // Get foot pressure data for both feet from FootModel
    footData = new float[getFrameHeight()][getFrameWidth() * 2];
    float[][] lf = leftfoot.getFoot(getFramesPerSecond(), getMaxPressure()*0.5f);
    float[][] rf = rightfoot.getFoot(getFramesPerSecond(), getMaxPressure());
    float v = 0f;

    for(int i = 0; i < getFrameHeight(); i++)
    {
      for(int j = 0; j < getFrameWidth() * 2; j++)
      {
        // Read pressure for each cell
        v = (j < getFrameWidth()) ? lf[i][j] : rf[getFrameHeight()-i-1][getFrameWidth()*2-j-1];

        // Square the pressure for realistic movement
        v = v*v*getMaxPressure()*getMaxPressure();
        footData[i][j] = (v < getMaxPressure()) ? v : getMaxPressure();
      }
    }

    this.currentFrame++;

    // Update the footfall preview/display if necessary
    if(displayfootfall != null)
      displayfootfall.updateDisplay(footData, getMaxPressure(), getFrameWidth(), getFrameHeight(), getFrameName());
  }
}